#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue May  8 10:56:07 2018

@author: Axelsen
"""


import sqlite3
import matplotlib.pyplot as plt
import time
import datetime
from mpl_toolkits.basemap import Basemap
import numpy as np
from scipy import stats
import pandas as pd
import loc_check as LC
from pylab import boxplot


#######################################################
## THE FOLLOWING PART EXTRACT DATA FROM THE DATABASE ##
#######################################################

def ExtractData(filepath,a,b,c,d,lowtime,hightime,maxspeed,minspeed,Analysis):
    global draughttime
    global useridDraught
    global draught
    global plotlat
    global plotlon
    global speeds
    global timestep
    global mmsi
    global navstat
    
    speeds = list()
    plotlat = list()
    plotlon = list()
    timestep = list()
    mmsi = list()
    navstat = list()
    draughttime = list()
    draught = list()
    useridDraught = list()

    conn = sqlite3.connect(filepath)
    cur = conn.cursor()
    
    # For nav-igation status, insert in '': 
    #portstatus = 'and (nav_status==1 or nav_status ==5)'
    
    SQLstring1 = "SELECT unixtime,sog,latitude,longitude,userid,\
        nav_status FROM %s WHERE longitude<= %s and latitude <= %s\
        and longitude >= %s and latitude >= %s and sog >= %s and \
        sog <= %s and unixtime >= %s and unixtime <= %s %s ORDER BY UNIXTIME \
        ASC" % ('LNG_fleet',str(a),str(b),str(c),str(d),str(minspeed),\
        str(maxspeed),str(lowtime),str(hightime),'')
    
    # Extract data from database:
    A = time.time()
    with conn:
        cur = conn.cursor()  
        if Analysis == 1:
            cur.execute(SQLstring1)
        VesselData = cur.fetchall()
        
        for i in range(0,len(VesselData)):
            Datastrip = VesselData[i]
            timestep.append(Datastrip[0])
            speeds.append(Datastrip[1])
            plotlat.append(Datastrip[2])
            plotlon.append(Datastrip[3])        
            mmsi.append(Datastrip[4])
            navstat.append(Datastrip[5])      
    
    cur.close()
    
    conn = sqlite3.connect(filepath)
    cur = conn.cursor()
#    
#    SQLstring2 = "SELECT unixtime,draught,userid from %s where \
#        unixtime >= %s and unixtime <= %s" % ('LNG_fleet5',str(lowtime),\
#        str(hightime))
#    
##    # Extract draught data from database:
#    with conn:
#        cur = conn.cursor()
#        if Analysis == 1:
#            cur.execute(SQLstring2)
#        draughtdata = cur.fetchall()
#    
#    for i in range(0,len(draughtdata)):
#        draughtstrip = draughtdata[i]
#        if draughtstrip[1]/10 > 5:
#            draughttime.append(draughtstrip[0])
#            draught.append(draughtstrip[1]/10)
#            useridDraught.append(draughtstrip[2])
            
    print('Database extraction time is: ' + str(time.time()-A) + ' s')
    return plotlon, plotlat


##################################################
## THE FOLLOWING PART PLOTS HISTOGRAM FOR SPEED ##
##################################################

def SpeedHistogram():
    plt.figure()
    hist, bins = np.histogram(speeds, bins=25)
    width = 0.7 * (bins[1] - bins[0])
    center = (bins[:-1] + bins[1:]) / 2
    plt.bar(center, hist, align='center', width=width)
    plt.show()

    plt.figure()
    plt.title('')
    histval1, binsval = np.histogram(speeds, bins=25)
    histval = histval1/sum(histval1)
    width = 0.7 * (binsval[1] - binsval[0])
    center = (binsval[:-1] + binsval[1:]) / 2
    plt.bar(center, histval, align='center', width=width)
    plt.xlabel('Speed [knots]')
    plt.ylabel('Fraction of time')
    plt.show()
    
    
    messages = len(speeds)
    unique_mmsi = len(set(mmsi))
    average_speed = np.mean(speeds)
    std_speed = np.std(speeds)
    
    print('The number of messages in the time period is: ' + str(messages))                
    print('The number of unique MMSI in the time period is: ' + str(unique_mmsi))
    print('The average speed in the time period is: ' + str(average_speed) +' knots')
    print('The standard deviation in the time period is: ' + str(std_speed) +' knots')
    
    
##################################################
## THE FOLLOWING PART DOES THE POLYGON ANALYSIS ##
##################################################
        
def PolygonAnalysis(lowtime,hightime):
    # Creating DataFram for analytical easiness           
    df = pd.DataFrame({'Speed': speeds, 'MMSI': mmsi, 'Unixtime': timestep, \
                             'Lat': plotlat, 'Lon': plotlon})

    polygons = LC.generate_polygons()
    
    #Creating empty column in the dataframe
    df['Zone'] = pd.Series({'Zone': range(len(df['MMSI']))})

    #Taking time:
    A = time.time()
    for i in range(0,len(plotlat)):
        if LC.point_inside_polygon(plotlon[i],plotlat[i],polygons[0]):
            df.set_value(i,'Zone','Atlantic')
        elif LC.point_inside_polygon(plotlon[i],plotlat[i],polygons[1]):
            df.set_value(i,'Zone','Pacific') #East Pacific
        elif LC.point_inside_polygon(plotlon[i],plotlat[i],polygons[2]):
            df.set_value(i,'Zone','Pacific') #West Pacific
        elif LC.point_inside_polygon(plotlon[i],plotlat[i],polygons[3]):
            df.set_value(i,'Zone','Indian Ocean')
        elif LC.point_inside_polygon(plotlon[i],plotlat[i],polygons[4]):
            df.set_value(i,'Zone','Mediterranean')
        elif LC.point_inside_polygon(plotlon[i],plotlat[i],polygons[5]):
            df.set_value(i,'Zone','North Sea')    
        elif LC.point_inside_polygon(plotlon[i],plotlat[i],polygons[6]):
            df.set_value(i,'Zone','SE Asia')
        elif LC.point_inside_polygon(plotlon[i],plotlat[i],polygons[7]):
            df.set_value(i,'Zone','Oceania')
        else: 
            df.set_value(i,'Zone','Outside Zones')
    print('Polygon iteration time is: ' + str(time.time()-A) + ' s')      
    
    #Defining dataframes for each zone
    Atlantic = df[df['Zone'] == 'Atlantic']
    Pacific = df[df['Zone'] == 'Pacific']
    IndianOcean = df[df['Zone'] == 'Indian Ocean']
    Mediterranean = df[df['Zone'] == 'Mediterranean']
    NorthSea = df[df['Zone'] == 'North Sea']
    SEAsia = df[df['Zone'] == 'SE Asia']
    Oceania = df[df['Zone'] == 'Oceania']    
    
    Timestamps = LC.get_timevector(lowtime,hightime)
    #print(Timestamps)
    dates = [datetime.datetime.fromtimestamp(u).strftime('%d/%m/%Y') for u in Timestamps]
    #print(dates)
    
    #Filter data for monthly basis
    A = time.time()
    speeds_monthly_Atlantic,monthly_stdev_Atlantic,unique_vessels_monthly_Atlantic,interval_Atlantic \
        = LC.monthly_filter(Atlantic,Timestamps)
    speeds_monthly_Pacific,monthly_stdev_Pacific,unique_vessels_monthly_Pacific,interval_Pacific \
        = LC.monthly_filter(Pacific,Timestamps)
    speeds_monthly_IndianOcean,monthly_stdev_IndianOcean,unique_vessels_monthly_IndianOcean,interval_IndianOcean \
        = LC.monthly_filter(IndianOcean,Timestamps)
    speeds_monthly_Medittarnean,monthly_stdev_Mediterranean,unique_vessels_monthly_Medittarnean,interval_Medittarnean \
        = LC.monthly_filter(Mediterranean,Timestamps)
    speeds_monthly_NorthSea,monthly_stdev_NorthSea,unique_vessels_monthly_NorthSea,interval_NorthSea \
        = LC.monthly_filter(NorthSea,Timestamps)
    speeds_monthly_SEAsia,monthly_stdev_SEAsia,unique_vessels_monthly_SEAsia,interval_SEAsia \
        = LC.monthly_filter(SEAsia,Timestamps)
    speeds_monthly_Oceania,monthly_stdev_Oceania,unique_vessels_monthly_Oceania,interval_Oceania \
        = LC.monthly_filter(Oceania,Timestamps)
    speeds_monthly_World,monthly_stdev_World,unique_vessels_monthly_World,interval_World \
        = LC.monthly_filter(df,Timestamps)
    print('Monthly filtering time is: ' + str(time.time()-A) + ' s') 
    
#    Percentage of fleet observed in zone each month
    prcentage_Atlantic = LC.percentageMonthly(unique_vessels_monthly_Atlantic,
                                              unique_vessels_monthly_World)
    prcentage_Pacific = LC.percentageMonthly(unique_vessels_monthly_Pacific,
                                              unique_vessels_monthly_World)
    prcentage_IndianOcean = LC.percentageMonthly(unique_vessels_monthly_IndianOcean,
                                              unique_vessels_monthly_World)
    prcentage_Medittarnean = LC.percentageMonthly(unique_vessels_monthly_Medittarnean,
                                              unique_vessels_monthly_World)
    prcentage_NorthSea = LC.percentageMonthly(unique_vessels_monthly_NorthSea,
                                              unique_vessels_monthly_World)
    prcentage_SEAsia = LC.percentageMonthly(unique_vessels_monthly_SEAsia,
                                              unique_vessels_monthly_World)
    prcentage_Oceania = LC.percentageMonthly(unique_vessels_monthly_Oceania,
                                              unique_vessels_monthly_World)
#        
    #Plotting the vessel distribution    
    timeperiod = list(range(0,(len(Timestamps)-1)))
    plt.style.use('bmh')
    
    #Plotting number of unique vessels
    fig, ax = plt.subplots(figsize=(12,5))
    #plt.title('Number of unique vessels in zone')
    plt.xlabel ('Months from %s to %s' % (str(dates[0]),str(dates[len(dates)-1])))
    plt.ylabel('Vessels in zone')
    ax.plot(timeperiod,unique_vessels_monthly_Atlantic,label = 'Atlantic')
    ax.plot(timeperiod,unique_vessels_monthly_Pacific,label = 'Pacific')
    ax.plot(timeperiod,unique_vessels_monthly_IndianOcean, label = 'Indian Ocean')
    ax.plot(timeperiod,unique_vessels_monthly_Medittarnean, label = 'Mediterranean Ocean')
    ax.plot(timeperiod,unique_vessels_monthly_NorthSea,label = 'North Sea')
    ax.plot(timeperiod,unique_vessels_monthly_SEAsia, label = 'South East Asia')
    ax.plot(timeperiod,unique_vessels_monthly_Oceania, label = 'Oceania')
    ax.plot(timeperiod,unique_vessels_monthly_World, label = 'World')
    #Placeing the legend outside the plot box:
    legend = ax.legend(bbox_to_anchor=(0., 1.02, 1., .102),ncol=4,loc=3,mode="expand",borderaxespad=0.)
    frame = legend.get_frame()
    frame.set_facecolor('0.90')
    for label in legend.get_lines():
        label.set_linewidth(1)
    plt.show()
    
    #Percentage of fleet observed in zone each month
    fig, ax = plt.subplots(figsize=(12,5))
    xaxis=np.arange(0,12,1)
    plt.xticks(xaxis)
    #plt.title('Percentage of fleet observed in zone each month')
    plt.xlabel ('Months from %s to %s' % (str(dates[0]),str(dates[len(dates)-1])))
    plt.ylabel('Percentage [%]')
    ax.plot(timeperiod,prcentage_Atlantic,label = 'Atlantic')
    ax.plot(timeperiod,prcentage_Pacific,label = 'Pacific')
    ax.plot(timeperiod,prcentage_IndianOcean, label = 'Indian Ocean')
    ax.plot(timeperiod,prcentage_Medittarnean, label = 'Mediterranean Ocean')
    ax.plot(timeperiod,prcentage_NorthSea,label = 'North Sea')
    ax.plot(timeperiod,prcentage_SEAsia, label = 'South East Asia')
    ax.plot(timeperiod,prcentage_Oceania, label = 'Oceania')
    #Placeing the legend outside the plot box:
    legend = ax.legend(bbox_to_anchor=(0., 1.02, 1., .102),ncol=4,loc=3,mode="expand",borderaxespad=0.)
    frame = legend.get_frame()
    frame.set_facecolor('0.90')
    for label in legend.get_lines():
        label.set_linewidth(1)
    plt.show()    
    
    #Plotting mean speed of vessels
    fig, ax = plt.subplots(figsize=(12,5))
    #plt.title('Average speed per month per zone')
    plt.xlabel ('Months from %s to %s' % (str(dates[0]),str(dates[len(dates)-1])))
    plt.ylabel('Speed [knots]')
    ax.plot(timeperiod,speeds_monthly_Atlantic,label = 'Atlantic')
    ax.plot(timeperiod,speeds_monthly_Pacific,label = 'Pacific')
    ax.plot(timeperiod,speeds_monthly_World, label = 'World')
    #Placeing the legend outside the plot box:
    legend = ax.legend(bbox_to_anchor=(0., 1.02, 1., .102),ncol=3,loc=3,mode="expand",borderaxespad=0.)
    frame = legend.get_frame()
    frame.set_facecolor('0.90')
    for label in legend.get_lines():
        label.set_linewidth(1)
    plt.show() 
    
    #Plotting world speed with standard deviation
    fig, ax = plt.subplots(figsize=(12,5))
    plt.xlabel ('Months from %s to %s' % (str(dates[0]),str(dates[len(dates)-1])))
    plt.ylabel('Speed [knots]')
    boxplot(monthly_stdev_World,0,'')
    plt.show() 
    
    #Plotting mean message interval of vessels
    fig, ax = plt.subplots(figsize=(12,5))
    #plt.title('Average message interval')
    plt.xlabel ('Months from %s to %s' % (str(dates[0]),str(dates[len(dates)-1])))
    plt.ylabel('Time [hours]')
    #ax.plot(timeperiod,interval_Atlantic,label = 'Atlantic')
    #ax.plot(timeperiod,interval_Pacific,label = 'Pacific')
    ax.plot(timeperiod,interval_World, label = 'World')
    #Placeing the legend outside the plot box:
    legend = ax.legend(bbox_to_anchor=(0., 1.02, 1., .102),ncol=2,loc=3,mode="expand",borderaxespad=0.)
    frame = legend.get_frame()
    frame.set_facecolor('0.90')
    for label in legend.get_lines():
        label.set_linewidth(1)
    plt.show()  
    
    #SPeed histogram
    SpeedHistogram(Atlantic['Speed'].tolist(),'Atlantic')
    SpeedHistogram(Pacific['Speed'].tolist(),'Pacific')

    LC.plot_inside_polygons(Atlantic['Lon'].tolist(),Atlantic['Lat'].tolist())  
    
    return df        
    
######################################################
## THE FOLLOWING PART PLOTS HISTOGRAM OF SPEED DIST ##
######################################################

def DraughtHistogram():    
    plt.figure()
    histval1, binsval = np.histogram(draught, bins=20)
    histval = histval1/sum(histval1)
    width = 0.7 * (binsval[1] - binsval[0])
    center = (binsval[:-1] + binsval[1:]) / 2
    plt.bar(center, histval, align='center', width=width)
    plt.xlabel('Draught [meters]')
    plt.ylabel('Fraction of time')
    plt.show()
    
    draughts = len(draught)
    print('The number of draughts in the time period is: ' + str(draughts))
    
################################################
## THE FOLLOWING PARTS PLOTS POSITIONS ON MAP ##
################################################
    
def LocalMap():
    minlon = max(-180,min(plotlon)-5) #-5
    minlat = max(-90,min(plotlat)-5) #-5
    maxlon = min(180,max(plotlon)+5) #+5
    maxlat = min(90,max(plotlat)+5) #+5
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig=plt.figure(figsize=(18,18))
    fig.add_axes([0.1,0.1,0.8,0.8])
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,urcrnrlat=maxlat,\
            rsphere=(6378137.00,6356752.3142),\
            resolution='l',projection='cyl',\
            lat_0=lat0,lon_0=lon0,lat_ts = lat1)

    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgray',lake_color='white',zorder=0) #,zorder=0
    x, y = m(plotlon,plotlat) 
    #Ships:
    m.scatter(x,y,0.01,marker='o',c='black')
    #m.drawcoastlines()
    #plt.legend(handles=[data])
    #plt.title('Ship movements')
    #m.drawparallels(np.arange(-90,90,20),labels=[1,1,0,1])       
    #m.drawmeridians(np.arange(-180,180,20),labels=[1,1,0,1])
    #m.bluemarble()
    #plt.saveffig('/Users/PatrickAndreNaess/Desktop/PyPlots/current_plot.eps',\
    #            format='eps', dpi=1000)                 

############################################################################
## THE FOLLOWING PART CALCULATES INTERVALS OF MESSAGES AND PLOTS HISTOGRAM #
############################################################################
    
def SpeedForAnalysis():
    global logdiff
    global UpDiff
    global UpTime
    global UpSpeed
    global AnSpeed
    global AnTime
    global AnSteps
    global AnDiff
    global AnLat
    global AnLon
    logdiff = list()
    logdiff = np.diff(timestep)  
    UpDiff = list()
    UpSpeed = speeds
    UpTime = timestep
    UpDiff = logdiff
    UpLat = plotlat
    UpLon = plotlon
    
    for i in range(0,len(logdiff)):
        if logdiff[i] < 1:
            logdiff[i] = logdiff[i]
            UpSpeed[i+1] = 101010
            UpTime[i+1] = 101010
            UpDiff[i] = 101010

    AnSpeed = [x for x in UpSpeed if x != 101010] #AnDiff = Message intervals for analysis
    AnTime = [x for x in UpTime if x != 101010]
    AnLat = [x for x in UpLat if x != 101010]
    AnLon = [x for x in UpLon if x != 101010] #AnSpeed = Speed for analysis
    AnDiff = [x for x in UpDiff if x != 101010] #AnTime = Time for analysis
    AnDiff.append(1)
    AnSteps = list()
    for i in range(0,len(AnTime)):
        Unixconv = datetime.datetime.fromtimestamp(AnTime[i])
        AnSteps.append(Unixconv)

################################################
## THE FOLLOWING PART PLOTS INTERVAL FREQUENCY #
################################################  
        
    Count = list()
    Number = list()
    for i in range(0,50):
        Count.append(i)
        Number.append(AnDiff.count(i))
    
    xaxis=np.arange(0,50,5)
    plt.figure(figsize=(18,6))
    plt.plot(Count,Number)
    plt.grid(True)
    plt.xticks(xaxis)
    plt.title('Message intervals')
    plt.xlabel('Time between messages [seconds]')
    plt.ylabel('Number of messages')
    plt.show()   
    
    plt.figure(figsize=(18,6))
    plt.scatter(AnSteps,AnSpeed,c='k',s=0.1)
    plt.xlabel('Speed [knots]')
    plt.ylabel('Year')
    #plt.plot(IncSteps,AvSpeed,'k')
    plt.show()
    
    messages = len(speeds)
    unique_mmsi = len(set(mmsi))
    #average_speed = np.mean(speeds)
    #std_speed = np.std(speeds)
    
    print('The number of messages in the time period is: ' + str(messages))                
    print('The number of unique MMSI in the time period is: ' + str(unique_mmsi))
    #print('The average speed in the time period is: ' + str(average_speed) +' knots')
    #print('The standard deviation in the time period is: ' + str(std_speed) +' knots')
    
def DraughtAnalysis():    
    global AnDraught
    global AnCat
    global DSpeed
    global DTime    
    global Dcat
    global slope
    global intercept
    global r_value
    global p_value
    global std_err
    Drange = max(draught)-min(draught)
    Intervals = 3
    Dsteps = Drange/Intervals
    Dcat = [0]*len(draught)

    for i in range(0,len(draught)):
        for j in range(0,Intervals-1):
            if  min(draught) + Dsteps*j <= draught[i] <= min(draught)+(j+1)*Dsteps :
                Dcat[i] = j+1

    AnDraught = list()
    AnCat = list() 
    DSpeed = list()
    DTime = list()
    for j in range(0,len(Dcat)-1):
        for i in range(0,len(AnTime)):
        #if AnTime[i]
            if AnTime[i]>=draughttime[j] and AnTime[i]<draughttime[j+1]:
                AnDraught.append(draught[j])
                AnCat.append(Dcat[j])
                DSpeed.append(AnSpeed[i])
                DTime.append(AnTime[i])
    slope, intercept, r_value, p_value, std_err = stats.linregress(AnDraught,DSpeed) 
    
    draughts = len(draught)
    print('The number of draughts in the time period is: ' + str(draughts))
